/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1991, 1995    *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * Mach Operating System
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */
/*
 * OSF/1 Release 1.0
 */
/*
 * derived from cpuconf.c	4.14      (ULTRIX)  12/6/90;
 */


/***********************************************************************
 *
 * Modification History: cpuconf.c
 *
 * 30-oct-91	jac	Fixup Alpha switches from sync to BL6
 *
 * 28-oct-91	jac	Added nulladdrcpu routine. Used as null routine for
 *			functions which must return an address (and therefore
 *			may have a 64 bit return value)
 *
 * 03-Oct-91    ald     added flamingo support
 *
 * 12-sep-91    jac     added nexaddr routine reference
 *
 * 21-jun-91    jac     added laser/ruby  support
 *
 * 03-May-91    afd
 *      Alpha support.
 *
 * 10-May-91	Paul Grist
 *	Added support for 3max+/bigmax (DS_5000_300).
 *
 * 15-Oct-90	Randall Brown
 *	Added errlogging routines to cpusw for 3min.
 *
 * 09-Oct-90    jaw
 * 	merge in MM changes for rigel.
 *
 * 09-Oct-90    Paul Grist
 *      added startclock routine to cpuswitch for mipsmate, this
 *      fixes hangs after power-fails when the system needs to
 *      acess non-root disks, which are not spun up, the cases
 *      that were seen were swap on non-root and presto buffers.
 *
 * 01-Sep-90	sekhar
 *	added functions and stubs for print_consinfo interface.
 *      kn02_print_consinfo, kn02_log_errinfo 	- DS5000 (3MAX)
 *	kn220_print_consinfo, kn220_log_errinfo	- DS5500 (MIPSFAIR2)
 *	nullcpu stubs for other machines (both mips and vax).
 *
 * 31-Aug-90	Jim Paradis
 *	Added additional stubs for VAX9000 routines
 *
 * 03-Aug-90	rafiey (Ali Rafieymehr)
 *	Added support for VAX9000.
 *
 * 21-Jun-90	Fred L. Templin
 *	Added dummies for TURBOchannel data structures for case of
 *	DS5000 not defines. (Solution from afd).
 *
 * 20-Mar-90    Paul Grist
 *      Added MIPSMATE support (DS_5100).
 *
 * 30-Apr-90	Randall Brown
 *	Added new cpu entry for DS_5000_100.  Filled in the new values
 *	of the switch table for the MIPS systems. ( spls, intr, clock stuff)
 *
 * 06-Mar-90	afd
 *	Put mc146818startclocks() into cpu switch for pmax/3max.
 *
 * 18-Jan-90	robin
 *	Added kn220badaddr function to get badaddr to work with the
 *	way the KN220 memory intrrupts are cleared.
 *
 * 29-Dec-89	afd
 *	Added definitions for kn02erradr & kn02chksyn for when
 *	DS5000 not defined.
 *
 * 26-Dec-89	robin
 *	changed the kn220 write buffer routine used by 5500.
 *
 * 08-Dec-89	jaw
 *	fix 6200 entry from merge damage.
 *
 * 30-Nov-89    Paul Grist
 *      Added 8800 error logging routines as stubs for non-8800 
 *      VAXBI systems that will use biinit.c Did the same for
 *      ka6200 and ka6400.
 *
 * 14-Nov-89	gmm
 *	Remove kn5800_iscpu() and add kn5800_init_secondary().
 *
 * 30-Oct-89	afd
 *	Use kn01 cache flush routines for kn02 (DS5000 - 3max).
 *
 * 11-Aug-89	afd
 *	Set up 3MAX cpu struct in cpu switch table.
 *	
 * 10-Jul-89	burns
 *	For DS5800 moved several "vax" only fields into mips. Added
 *	the msize and cpuinit routines for afd. Added cache routines
 *	to the cpu switch for DS5800.
 *
 * 14-June-89	afd
 *	Fill in new HZ field in cpuswitch (used to be in param.c).
 *	hz, tick, tickadj are set in processor specific init routines.
 *
 * 23-May-89	darrell
 *	Merged VVAX support.
 *
 * 03-May-89	darrell
 *	Merged in VAX6400 support.
 *
 * 07-Apr-89	afd
 *	Created this file as a merged version of the old VAX cpuconf.c
 *	with new entries for MIPS based systems.  This file now supports
 *	both VAX and MIPS based systems.
 *
 **********************************************************************/


#include <sys/param.h>
#include <hal/cpuconf.h>
#include <machine/cpu.h>
#include <io/dec/uba/ubareg.h>

int	nocpu();
int	nullcpu();
vm_offset_t nulladdrcpu();

int	readtodr();
int 	writetodr();
int 	uICRdelay();
int 	uInoICRdelay();
int 	uIInoICRdelay();
int 	cVSnoICRdelay();
int 	uSSCdelay();
int	uRSSCdelay();
int	ssc_readtodr();
int	ssc_writetodr();

char    UNKNOWN_SYS_STRING[] = "Unknown system type";

#ifdef DECAXPPCI_33
extern int      ebp10_machcheck();
extern int      ebp10_badaddr();
extern int      ebp10_conf();
extern int      ebp10_init();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int      lca_map_io();
extern int	ebp10_dump_dev();
extern int	pc_ring_bell();
extern int	ebp10_config_secondary();
extern int	ebp10_init_secondary();
extern int	ebp10_getinfo();
char    ALPHA_AXPPCI_33_STRING[] = "Alpha PC AXPpci33  ";
#endif /* DECAXPPCI_33 */

#ifdef DEC2100_A50
extern int      kn25_machcheck();
extern int      kn25_badaddr();
extern int      kn25_conf();
extern int      kn25_init();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int      apecs_map_io();
extern int	kn25_dump_dev();
extern int	pc_ring_bell();
extern int	kn25_config_secondary();
extern int	kn25_init_secondary();
extern int	kn25_getinfo();
char    ALPHA_APECS_STRING[] = "AVANTI (EV4/APECS)";
#endif /* DEC2100_A50 */

#ifdef DEC1000
extern int      kn22a_machcheck(); /* 0x660, 0x670 */
extern int      kn22a_proccorr();  /* 0x620, 0x630 */
extern int      kn22a_badaddr();
extern int      kn22a_conf();
extern int      kn22a_init();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int      apecs_map_io();
extern int	kn22a_dump_dev();
extern int	pc_ring_bell();
extern int	kn22a_config_secondary();
extern int	kn22a_init_secondary();
extern int	kn22a_getinfo();
char    DEC1000_STRING[] = "DEC1000";
#endif /* DEC1000 */

#ifdef DEC2100_A500
extern int      kn450_machcheck();	/* 0x660, 0x670 */
extern int      kn450_proccorr();	/* 0x630 */
extern int      kn450_conf();
extern int      kn450_init();
extern int	kn450_memenable();
extern int      kn450_cachenble();
extern int      kn450_cachdisble();
extern int      kn450_flush_cache();
extern int      kn450_badaddr();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int	kn450_read_io_port();
extern int	kn450_write_io_port();
extern int	pc_ring_bell();
extern int	kn450_dump_dev();
extern int	kn450_getinfo();
extern int	kn450_getinfo(); 
extern int	kn450_config_secondary();
extern int	kn450_init_secondary();
extern int      kn450_mces_timeout();
char    DEC2100_A500_STRING[] = "DEC2100 A500";
#else  /* DEC2100_A500 */
#ifndef DEC2100_C500
extern int	kn450_config_io()  { return (0); }
#endif /* !DEC2100_C500 */
#endif /* DEC2100_A500 */

#ifdef DEC2100_C500
extern int      kn470_machcheck();	/* 0x660, 0x670 */
extern int      kn470_proccorr();	/* 0x630 */
extern int      kn470_conf();
extern int      kn470_init();
extern int	kn450_memenable();
extern int      kn450_cachenble();
extern int      kn450_cachdisble();
extern int      kn450_flush_cache();
extern int      kn470_badaddr();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int	kn450_read_io_port();
extern int	kn450_write_io_port();
extern int	pc_ring_bell();
extern int	kn450_dump_dev();
extern int	kn450_getinfo();
extern int	kn450_config_secondary();
extern int	kn450_init_secondary();
extern int	kn470_testpage();
extern int      kn450_mces_timeout();
char    DEC2100_C500_STRING[] = "DEC2100 C500";

#else /*  DEC2100_C500 */
extern int	kn470_config_io()  { return (0); }
#endif /* DEC2100_C500 */


#ifdef DEC_KN20AA
extern int      kn20aa_machcheck();
extern int      kn20aa_proccorr();	
extern int      kn20aa_conf();
extern int      kn20aa_init();
extern int	kn20aa_memenable();
extern int      kn20aa_cachenble();
extern int      kn20aa_cachdisble();
extern int      kn20aa_flush_cache();
extern int      kn20aa_badaddr();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int	kn20aa_read_io_port();
extern int	kn20aa_write_io_port();
extern int	kn20aa_dump_dev();
extern int	pc_ring_bell();
extern int	kn20aa_dump_dev();
extern int	kn20aa_getinfo();
extern int	kn20aa_testpage();
char    	KN20AA_STRING[] = "DEC - MODEL KN20AA";
#endif /* DEC_KN20AA  */

#ifdef ALPHAADU
extern int      adu_machcheck();
extern int      adu_memerr();
extern int      adu_crderr();
extern int      adu_memenable();
extern int      adu_conf();
extern int      adu_init();
extern int      adu_cachenble();
extern int      adu_cachdisble();
extern int      adu_flush_cache();
extern int      alpha_readtodr();
extern int      alpha_writetodr();
extern int      adu_delay();
extern int      adu_map_io();
char    ALPHA_ADU_STRING[] = "ALPHA ADU";
#endif /* ALPHAADU */

#ifdef DEC7000
extern int      ruby_machcheck();
extern int      kn7aa_proccorr();
extern int      kn7aa_syscorr();
extern int      lsb_memerr();
extern int      lsb_crderr();
extern int      lsb_memenable();
extern int      ruby_conf();
extern int      ruby_init();
extern int      ruby_cachenble();
extern int      ruby_cachdisble();
extern int      ruby_flush_cache();
extern int      ruby_badaddr();
extern int      ka_ruby_readtodr();
extern int      ka_ruby_writetodr();
extern int      ruby_map_io();
extern int	kn7aa_config_secondary();
extern int	kn7aa_init_secondary();
extern int	kn7aa_dump_dev();
extern int	kn7aa_getinfo();
extern int      kn7aa_mces_timeout();
extern vm_offset_t	ruby_nexaddr();
char    DEC7000_STRING[] = "DEC7000";
#endif /* DEC7000 */

#ifdef DEC21000
extern int      tlaser_machcheck();
extern int      tlaser_proccorr();
extern int      tlaser_syscorr();
extern int      lsb_memerr();
extern int      lsb_crderr();
extern int      tlsb_memenable();
extern int      tlaser_conf();
extern int      tlaser_init();
extern int      ruby_cachenble();
extern int      ruby_cachdisble();
extern int      ruby_flush_cache();
extern int      ruby_badaddr();
extern int      ka_tlaser_readtodr();
extern int      ka_tlaser_writetodr();
extern int      tlaser_map_io();
extern int	tlaser_config_secondary();
extern int	tlaser_init_secondary();
extern int	kn7aa_dump_dev();
extern int	kn7aa_getinfo();
extern vm_offset_t	tlaser_nexaddr();
extern int	tlaser_testpage();
extern int      tlaser_mces_timeout();
char    DEC21000_STRING[] = "DEC21000";
#endif /* DEC21000 */

#ifdef DEC4000
extern int      kn430_machcheck();	/* 0x660, 0x670 */
extern int      kn430_proccorr();	/* 0x630 */
extern int      cobra_conf();
extern int      cobra_init();
extern int      cobra_cachenble();
extern int      cobra_cachdisble();
extern int      cobra_flush_cache();
extern int      cobra_badaddr();
extern int      cobra_readtodr();
extern int      cobra_writetodr();
extern int      cobra_consprint();
extern int	kn430_config_secondary();
extern int	kn430_init_secondary();
extern int	kn430_dump_dev();
extern vm_offset_t	cobra_nexaddr();
extern int	kn430_getinfo();
char    DEC4000_STRING[] = "DEC4000";
#endif /* DEC4000 */

#ifdef DEC3000_500
extern int      kn15aa_machcheck();
extern int      kn15aa_memerr();
extern int      kn15aa_crderr();
extern int      kn15aa_memenable();
extern int      kn15aa_conf();
extern int      kn15aa_init();
extern int      kn15aa_cachenble();
extern int      kn15aa_cachdisble();
extern int      kn15aa_flush_cache();
extern int      kn15aa_badaddr();
extern int      kn15aa_readtodr();
extern int      kn15aa_writetodr();
extern int      kn15aa_map_io();
extern int	kn15aa_system_id();
extern int	kn15aa_dump_dev();
extern int	kn15aa_getinfo();
char    DEC_3000_500_STRING[] = "DEC3000 - M500";
#else
kn15aa_is_sandpiper() { return (0); }
kn15aa_set_ioslot() {}
kn15aa_read_ioslot() { return (0); }
kn15aa_read_ir() { return (0); }
#endif /* DEC3000_500 */

#ifdef DEC3000_300
extern int      kn16aa_machcheck();
extern int      kn16aa_memerr();
extern int      kn16aa_crderr();
extern int      kn16aa_memenable();
extern int      kn16aa_conf();
extern int      kn16aa_init();
extern int      kn16aa_cachenble();
extern int      kn16aa_cachdisble();
extern int      kn16aa_flush_cache();
extern int      kn16aa_badaddr();
extern int      kn16aa_readtodr();
extern int      kn16aa_writetodr();
extern int      kn16aa_map_io();
extern int 	kn16aa_system_id();
extern int	kn16aa_dump_dev();
extern int	kn16aa_getinfo();
char    DEC_3000_300_STRING[] = "DEC3000 - M300";
#endif /* DEC3000_300 */


#ifdef DEC2000_300
extern int      kn121_machcheck();
extern int      kn121_conf();
extern int      kn121_init();
extern int      kn121_cachenble();
extern int      kn121_cachdisble();
extern int      kn121_flush_cache();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int	kn121_read_io_port();
extern int	kn121_write_io_port();
extern int	kn121_dump_dev();
extern int	kn121_getinfo();
extern int	pc_ring_bell();
char    DEC_2000_300_STRING[] = "DEC2000 - Model 300";
#else
extern int	kn121_configure_io()  { return (0); }
#endif /*DEC2000_300*/

#ifdef DECAXPVME_64
extern int      ebv10_machcheck();
extern int      ebv10_badaddr();
extern int      ebv10_conf();
extern int      ebv10_init();
extern int      ds1386_readtodclk();
extern int      ds1386_writetodclk();
extern int      alpha_delay();
extern int      lca_map_io();
extern int	ebv10_dump_dev();
extern int	ebv10_config_secondary();
extern int	ebv10_init_secondary();
extern int	ebv10_getinfo();
char    DEC_AXPVME_64_STRING[] = "AXPvme SBC";
#endif /* DECAXPVME_64 */

#ifdef ALPHAPC64
extern int      eb64p_machcheck();
extern int      eb64p_badaddr();
extern int      eb64p_conf();
extern int      eb64p_init();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int      apecs_map_io();
extern int	eb64p_dump_dev();
extern int	pc_ring_bell();
extern int	eb64p_config_secondary();
extern int	eb64p_init_secondary();
extern int	eb64p_getinfo();
char    EB64P_STRING[] = "Alpha 21064 PCI Single Board Computer (AlphaPC64, EB64+, or equivalent)";
#endif /* ALPHAPC64 */

#ifdef EB66PLUS
extern int      eb66_machcheck();
extern int      eb66_badaddr();
extern int      eb66_conf();
extern int      eb66_init();
extern int      mc146818_readtodclk();
extern int      mc146818_writetodclk();
extern int      alpha_delay();
extern int      lca_map_io();
extern int	eb66_dump_dev();
extern int	pc_ring_bell();
extern int	eb66_config_secondary();
extern int	eb66_init_secondary();
extern int	eb66_getinfo();
char    EB66_STRING[] = "Alpha 21066 PCI Single Board Computer (EB66+ or equivalent)";
#endif /* EB66PLUS */

struct cpusw	cpusw[] =
{
#ifdef DEC2000_300
        { DEC_2000_300,		kn121_machcheck,	nullcpu,
          nullcpu,		nullcpu,		nullcpu,
          nullcpu,              kn121_conf,             nullcpu,
          nullcpu,              nullcpu,                nocpu,
          mc146818_readtodclk,	mc146818_writetodclk,	nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  kn121_read_io_port,	kn121_write_io_port,	pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  kn121_dump_dev,	kn121_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn121_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          DEC_2000_300_STRING,	100 /*HZ*/,		0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC2000_300 */

#ifdef DECAXPPCI_33
        { DEC_AXPPCI_33,	ebp10_machcheck, 	nullcpu,
          nullcpu,		nullcpu,		nullcpu,
          nullcpu,		ebp10_conf,		nullcpu,
          nullcpu,              nullcpu,                ebp10_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              lca_map_io,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		ebp10_config_secondary,	ebp10_init_secondary,
	  ebp10_dump_dev,	ebp10_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          ebp10_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          ALPHA_AXPPCI_33_STRING, 100 /*HZ*/,		0 /*flags*/,

	  SPLDEVRT /* BUS Probe SPL */,

          },
#endif /* DECAXPPCI_33 */

#ifdef DEC2100_A50
        { DEC_2100_A50,		kn25_machcheck,	nullcpu,
          nullcpu,		nullcpu,		nullcpu,
          nullcpu,		kn25_conf,		nullcpu,
          nullcpu,              nullcpu,                kn25_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              apecs_map_io,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		kn25_config_secondary,	kn25_init_secondary,
	  kn25_dump_dev,	kn25_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn25_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          ALPHA_APECS_STRING,	100 /*HZ*/,		0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC2100_A50 */

#ifdef DEC1000
        { DEC_1000,		kn22a_machcheck,	kn22a_proccorr,
          kn22a_proccorr,	nullcpu,		nullcpu,
          nullcpu,		kn22a_conf,		nullcpu,
          nullcpu,              nullcpu,                kn22a_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              apecs_map_io,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		kn22a_config_secondary,	kn22a_init_secondary,
	  kn22a_dump_dev,	kn22a_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn22a_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          DEC1000_STRING,	100 /*HZ*/,		0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC1000 */

#ifdef DEC2100_A500
        { DEC_2100_A500,	kn450_machcheck, 	nullcpu,
          kn450_proccorr,	kn450_memenable,	nullcpu,
          nullcpu,		kn450_conf,		nullcpu,
          nullcpu,              nullcpu,                kn450_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              nullcpu,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  kn450_read_io_port,	kn450_write_io_port,	pc_ring_bell,
	  nullcpu,		kn450_mces_timeout,		nullcpu,
	  nullcpu,		kn450_config_secondary,	kn450_init_secondary,
	  kn450_dump_dev,	kn450_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn450_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          DEC2100_A500_STRING,	 100 /*HZ*/,		 0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC2100_A500 */

#ifdef DEC2100_C500
        { DEC_2100_C500,	kn470_machcheck, 	nullcpu,
          kn470_proccorr,	kn450_memenable,	nullcpu,
          nullcpu,		kn470_conf,		nullcpu,
          nullcpu,              nullcpu,                kn470_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              nullcpu,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  kn450_read_io_port,	kn450_write_io_port,	pc_ring_bell,
	  kn470_testpage,	kn450_mces_timeout,		nullcpu,
	  nullcpu,		kn450_config_secondary,	kn450_init_secondary,
	  kn450_dump_dev,	kn450_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn470_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          DEC2100_C500_STRING,	 100 /*HZ*/,		 0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC2100_C500 */

#ifdef DEC_KN20AA
        { ALPHA_KN20AA,       	kn20aa_machcheck, 	nullcpu,
          kn20aa_proccorr,    	kn20aa_memenable,	nullcpu,
          nullcpu,		kn20aa_conf,		nullcpu,
          nullcpu,              nullcpu,                kn20aa_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              nullcpu,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  kn20aa_read_io_port,	kn20aa_write_io_port,	pc_ring_bell,
	  kn20aa_testpage,	nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  kn20aa_dump_dev,	kn20aa_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn20aa_init,
          0,	 		0,			10000,
          KN20AA_STRING,	100, /*HZ*/		0, /*flags*/

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC_KN20AA */

#ifdef DEC7000
        { DEC_7000,             ruby_machcheck,        kn7aa_syscorr,
          kn7aa_proccorr,       lsb_memenable,          nullcpu,
          nullcpu,              ruby_conf,              nullcpu,
          nullcpu,              nullcpu,                ruby_badaddr,
          ka_ruby_readtodr,     ka_ruby_writetodr,      nullcpu,
          nullcpu,              ruby_map_io,            nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          ruby_nexaddr,         nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		kn7aa_mces_timeout,		nullcpu,
	  nullcpu,           	kn7aa_config_secondary, kn7aa_init_secondary,
	  kn7aa_dump_dev,	kn7aa_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          ruby_init, /* could be nullcpu */
          0,                    0,                      10000 /* drift */,
	  DEC7000_STRING,	100 /*HZ*/,
	  SCS_START_SYSAPS/*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },

#endif /* DEC7000 */

#ifdef DEC21000
        { DEC_21000,           tlaser_machcheck,       tlaser_syscorr,
          tlaser_proccorr,	tlsb_memenable,		nullcpu,
          nullcpu,              tlaser_conf,            nullcpu,
          nullcpu,              nullcpu,                ruby_badaddr,
          ka_tlaser_readtodr,     ka_tlaser_writetodr,      nullcpu,
          nullcpu,              tlaser_map_io,          nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          tlaser_nexaddr,	nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  tlaser_testpage,	tlaser_mces_timeout,		nullcpu,
	  nullcpu,           	tlaser_config_secondary, tlaser_init_secondary,
	  kn7aa_dump_dev,	kn7aa_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          tlaser_init, /* could be nullcpu */
          0,                    0,                      10000 /* drift */,
	  DEC21000_STRING,	100 /*HZ*/,
	  SCS_START_SYSAPS/*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },

#endif /* DEC21000 */

#ifdef DEC4000
        { DEC_4000,             kn430_machcheck,        nullcpu,
          kn430_proccorr,       nullcpu,		nullcpu,
          nullcpu,              cobra_conf,             nullcpu,
          nullcpu,              nullcpu,                cobra_badaddr,
          cobra_readtodr,       cobra_writetodr,        nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,	        nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		kn430_config_secondary,	kn430_init_secondary,
	  kn430_dump_dev,	kn430_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          cobra_init, /* could be nullcpu */
          0,                    0,                      10000 /* drift */,
	  DEC4000_STRING,	100 /*HZ*/,		0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC4000 */

#ifdef DEC3000_500
        { DEC_3000_500,       	kn15aa_machcheck,     	kn15aa_memerr,
          kn15aa_crderr,      	kn15aa_memenable,     	nullcpu,
          nullcpu,              kn15aa_conf,          	nullcpu,
          nullcpu,              nullcpu,	      	kn15aa_badaddr,
          kn15aa_readtodr,    	kn15aa_writetodr,	nullcpu,
          nullcpu,              kn15aa_map_io,	        nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  kn15aa_system_id,	nullcpu,		nullcpu,
	  kn15aa_dump_dev,	kn15aa_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn15aa_init, /* can not be nullcpu */
          0,                    0,                      10000 /* drift */,
	  DEC_3000_500_STRING, 	100 /*HZ*/,		0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC3000_500 */

#ifdef DEC3000_300
        { DEC_3000_300,         kn16aa_machcheck,       kn16aa_memerr,
          kn16aa_crderr,        kn16aa_memenable,       nullcpu,
          nullcpu,              kn16aa_conf,            nullcpu,
          nullcpu,              nullcpu,                kn16aa_badaddr,
          kn16aa_readtodr,      kn16aa_writetodr,       nullcpu,
          nullcpu,              kn16aa_map_io,          nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,              	nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  kn16aa_system_id,	nullcpu,		nullcpu,
	  kn16aa_dump_dev,	kn16aa_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          kn16aa_init, /* can not be nullcpu */
          0,                    0,                      10000 /* drift */,
	  DEC_3000_300_STRING, 100 /*HZ*/,              0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* DEC3000_300 */

#ifdef DECAXPVME_64
        { DEC_AXPVME_64,        ebv10_machcheck,	nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		ebv10_conf,		nullcpu,
	  nullcpu,              nullcpu,                ebv10_badaddr,
	  ds1386_readtodclk,	ds1386_writetodclk,	alpha_delay,
	  nullcpu,              lca_map_io,		nullcpu,
	  nullcpu,              nullcpu,                nullcpu,
	  nulladdrcpu,          nullcpu,		nullcpu,
	  nullcpu,              nullcpu,		nullcpu,
	  nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,	  	ebv10_config_secondary, ebv10_init_secondary,
	  ebv10_dump_dev,       ebv10_getinfo,
/* end of cpusw common area */
/* Specific Routines */
	  ebv10_init,
	  0 /*pc_umsize*/,	0 /*pc_haveubasr*/,	10000 /* drift */,
	  DEC_AXPVME_64_STRING, 100 /*HZ*/,		0 /*flags*/,

	  SPLDEVRT /* BUS Probe SPL */,

          },
#endif /* DECAXPVME_64 */

#if ALPHAPC64
        { EB64_PLUS,		eb64p_machcheck,	nullcpu,
          nullcpu,		nullcpu,		nullcpu,
          nullcpu,		eb64p_conf,		nullcpu,
          nullcpu,              nullcpu,                eb64p_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              apecs_map_io,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		eb64p_config_secondary,	eb64p_init_secondary,
	  eb64p_dump_dev,	eb64p_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          eb64p_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          EB64P_STRING,		 100 /*HZ*/,		 0 /*flags*/,

	  SPLEXTREME /* BUS Probe SPL */,

          },
#endif /* ALPHAPC64 */

#if EB66PLUS
        { LCA_EB66,		eb66_machcheck,		nullcpu,
          nullcpu,		nullcpu,		nullcpu,
          nullcpu,		eb66_conf,		nullcpu,
          nullcpu,              nullcpu,                eb66_badaddr,
          mc146818_readtodclk,	mc146818_writetodclk,	alpha_delay,
          nullcpu,              lca_map_io,		nullcpu,
          nullcpu,              nullcpu,                nullcpu,
          nulladdrcpu,          nullcpu,		nullcpu,
          nullcpu,              nullcpu,		nullcpu,
	  nullcpu,		nullcpu,		pc_ring_bell,
	  nullcpu,		nullcpu,		nullcpu,
	  nullcpu,		eb66_config_secondary,	eb66_init_secondary,
	  eb66_dump_dev,	eb66_getinfo,
/* end of cpusw common area */
/* Specific Routines */
          eb66_init,
          0 /*pc_umsize*/,	 0 /*pc_haveubasr*/,	 10000 /* drift */,
          EB66_STRING,		 100 /*HZ*/,		 0 /*flags*/,

	  SPLDEVRT /* BUS Probe SPL */,

          },
#endif /* EB66PLUS */

	/*
	 * We have to be able to find the end of the table
	 */
    {	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,			0,			0,
	0,
	 }
};


extern struct cpusw *cpup;

/*
 * The following was added for network management software.
 * Purpose is to provide a friendly system id string.
 */
char *
get_system_type_string ()
{
    return (cpup->system_string);
}

/*
 * When this routine is called, we are doing something wrong.
 */
nocpu()
{
	panic("nocpu: unimplemented cpu");
}

/*
 * null routine to pass back a success since this cpu type
 * doesn't need one of these routines.
 */
nullcpu()
{
	return(0);
}

/*
 * nulladdrcpu() is used to as a stub for those CPUs that don't need
 * to use a routine that otherwise must return an addresss
*/
vm_offset_t nulladdrcpu()
{
    static int once = 0;
    if (once == 0) {
	printf("nulladdrcpu called\n");
	once++;
    }
	return(0);
}

