/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1991, 1995    *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
#pragma ident "@(#)$RCSfile: lockinfo.c,v $ $Revision: 1.1.157.2 $ (DEC) $Date: 1995/05/31 20:06:27 $"

#include <kern/lock.h>
#include <machine/cpu.h>
#include <unix_locks.h>
#include <net/net_globals.h>

#include "rt_sem.h"
#include "rt_ipc.h"
#include "rt_csem.h"
#include "te.h"
#include "tu.h"
#include "xna.h" 
#include "mfa.h"
#include "faa.h"
#include "packetfilter.h"
#include "gwscreen.h"
#include "pty.h"
#include "rpty.h"
#include "dli.h"
#include "dlb.h"
#include  "dlpi.h"
#include "dec_audit.h"
#include "siop.h"
#include "xza.h"
#include "aha.h"
#include "pza.h"
#include "tza.h"
#include "isp.h"
#include "psiop.h"
#include "xcr.h"
#include "tra.h"
#include "fta.h"
#include "trsrcf.h"
#include "mlog.h"
#include "cdfs.h"
#include "ffm_fs.h"
#include "tcds.h"
#include "streams.h"
#include "strifnet.h"
#include "ci.h"
#include "uq.h"
#include "msi.h"
#include "ldtty.h"
#include "pci.h"
#include "msfs.h"

/*
 * Define lockmode here so that we can have a different initial value
 * for the SAS kernel.  When we make osf_boot smart enough to read MFS,
 * then this definition could go back to lock.c.
 */
#include "sas.h"
#if SAS > 0
long lockmode =  0; /* 0: uni, 1: rt, 2: smp, 3: rt/smp, 4: rt/smp/dbg/stats */
#else
long lockmode = -1; /* to be determined at bootstrap time */
#endif /* SAS > 0 */

#if MACH_SLOCKS
#define SCC(xxx) xxx
#else
#define SCC(xxx)
#endif

#if UNIX_LOCKS
#define UCC(xxx) xxx
#else
#define UCC(xxx)
#endif

#if NETSYNC_LOCK
#define NCC(xxx) xxx
#else
#define NCC(xxx)
#endif

#define LI(name, extlip, minspl, flags) { \
	extern struct lockinfo *extlip; \
	lip->li_name = name; \
	lip->li_count = 0; \
	lip->li_flgspl = (minspl) | (flags); \
	extlip = lip++; \
}
#define SLI(a,b,c,d) SCC(LI(a,b,c,d))
#define ULI(a,b,c,d) UCC(LI(a,b,c,d))
#define NLI(a,b,c,d) NCC(LI(a,b,c,d))

#define LI_DISABLE_HIERARCHY_CHECKING(p) (p)->li_flgspl &= ~LI_DOHIER
#define SLI_DISABLE_HIERARCHY_CHECKING(p) SCC(LI_DISABLE_HIERARCHY_CHECKING(p))
#define ULI_DISABLE_HIERARCHY_CHECKING(p) UCC(LI_DISABLE_HIERARCHY_CHECKING(p))
#define NLI_DISABLE_HIERARCHY_CHECKING(p) NCC(LI_DISABLE_HIERARCHY_CHECKING(p))

#define DEFAULT (LI_VALID | LI_DOHIER) /* default lockinfo flags */
#define ORDERED (DEFAULT | LI_ORDERED)
#define ORDNEXT (DEFAULT | LI_ORDNEXT)

#define ULI_NOCHECK(name, extlip) { \
	ULI(name, extlip, SPLNONE, DEFAULT); \
	ULI_DISABLE_HIERARCHY_CHECKING(extlip); \
}

extern int lockinfo_count;
extern struct lockinfo *lockinfo_table;
extern struct lockstats *lockstats_table[];


/*
 * The following function contains lockinfo specs for initializing the
 * complex and simple lock data structures associated with lock debugging
 * and statistics.  The order of the LI() macro invocations determines the
 * relative locking hierarchy positions between classes of locks, with specs
 * nearer the beginning of the function referring to higher (or outermost)
 * locks and specs nearer the end of the function referring to lower (or
 * innermost) locks.  To add a new lockinfo spec, follow these steps:
 *
 *	1) Determine where the class of locks fits in below, and add a new
 *		LI() line in the function below specifying the class name,
 *		the name of an external lockinfo structure pointer, the
 *		minimum required spl level for lock acquisition, and the
 *		default trace flag constant.  The suggested class naming
 *		convention is to use the name of the global lock instance
 *		for single-lock classes or <structure_name>.<field_name>
 *		for classes of locks which are fields in other structures.
 *		If the associated lock instances are conditionally compiled,
 *		use the SLI(), ULI(), or NLI() macro in place of LI() as
 *		appropriate.
 *
 *	2) Add the global (struct lockinfo *) variable to the source file
 *		related to the locking class.  Use conditionally compiled
 *		declarations (decl_simple_lock_info/udecl_simple_lock_info)
 *		if the associated lock instances are conditionally compiled.
 *
 *	3) Replace the associated lock_init() or simple_lock_init() calls
 *		with lock_setup() or simple_lock_setup() calls naming the
 *		global (struct lockinfo *) variable as the 2nd argument.
 *
 *	4) If the lock class is a field of a data structure which is
 *		dynamically allocated and freed, add a complementary call
 *		to lock_terminate() or simple_lock_terminate() just before
 *		the data structure is freed.
 */

lockinfo_bootstrap(lip_start)
struct lockinfo *lip_start;
{
	register struct lockinfo *lip;
	register struct lockstats *lsp;
	register int i;

	lip = lip_start;
	lip->li_name = "(unknown_simple_lock)";
	lip->li_count = 0;
	lip->li_flgspl = 0;
	lip++;
	lip->li_name = "(unknown_complex_lock)";
	lip->li_count = 0;
	lip->li_flgspl = 0;
	lip++;

	/*********************************************/
	/* start of lockinfo specs for complex locks */
	/*********************************************/

	LI("sys_exitp_lock", sys_exitp_lockinfo, SPLNONE, DEFAULT);
	LI("fdfs_lock", fdfs_lockinfo, SPLNONE, DEFAULT);
	LI("cfg_subsys_lock", cfg_subsys_lockinfo, SPLNONE, ORDERED);
	LI("subsys_tbl_lock", subsys_tbl_lockinfo, SPLNONE, DEFAULT);
	LI("inode.i_io_lock", i_io_lockinfo, SPLNONE, ORDERED);
	LI_DISABLE_HIERARCHY_CHECKING(i_io_lockinfo);
	LI("nfs_daemon_lock", nfs_daemon_lockinfo, SPLNONE, DEFAULT);
	ULI("procfs_global_lock", procfs_global_lockinfo, SPLNONE, DEFAULT);
	LI("procfs.pr_trace_lock", procfs_trace_lockinfo, SPLNONE, DEFAULT);
	ULI("procnode.prc_ioctl_lock", procnode_ioctl_lockinfo,SPLNONE,DEFAULT);
	LI("semidq_lock", semidq_lockinfo, SPLNONE, DEFAULT);
	LI("semid_lock", semid_lockinfo, SPLNONE, DEFAULT);
	LI("undo_lock", undo_lockinfo, SPLNONE, DEFAULT);
	LI("msgidq_lock", msgidq_lockinfo, SPLNONE, DEFAULT);
	LI("msgid_lock", msgid_lockinfo, SPLNONE, DEFAULT);
	LI("utask.uu_addr_lock", u_addr_lockinfo, SPLNONE, DEFAULT);
	LI("vm_umap.vm_lock", u_map_lockinfo, SPLNONE, DEFAULT);
	NLI("unp_mutex_lock", unp_mutex_li, SPLNONE, DEFAULT);
	ULI("chtabhead_lock", chtabhead_lockinfo, SPLNONE, DEFAULT);
	NLI("global_raw_lock", global_raw_li, SPLNONE, DEFAULT);
	NLI("domain_lock", domain_li, SPLNONE, DEFAULT);
#if DLI > 0 || DLB > 0
	NLI("dli_input_lock", dli_input_li, SPLNONE, DEFAULT);
#endif
	NLI("socklocks.sock_lock", sock_li, SPLNONE, ORDERED);
	NLI("socklocks.snd_lock", snd_li, SPLNONE, DEFAULT);
	NLI("socklocks.rcv_lock", rcv_li, SPLNONE, DEFAULT);
	LI("pgrphash_lock", pgrphash_lockinfo, SPLNONE, DEFAULT);
	ULI("proc_relation_lock", proc_relation_lockinfo, SPLNONE, DEFAULT);
	ULI("pgrp.pg_lock", pgrp_lockinfo, SPLNONE, ORDERED);
	ULI("sysdev_write_lock", sysdev_write_lockinfo, SPLNONE, DEFAULT);
	LI("cm_lock", cm_lockinfo, SPLNONE, DEFAULT);
	LI("shmidq_lock", shmidq_lockinfo, SPLNONE, DEFAULT);
	LI("shmid_lock", shmid_lockinfo, SPLNONE, DEFAULT);
	LI("lk_lmf", lk_lmf_lockinfo, SPLNONE, DEFAULT);
	NLI("pr_dom_lock", pr_dom_li, SPLNONE, DEFAULT);
	/* inp_lock must follow socket lock */
	NLI("tcb.inp_lock", inp_tcphead_li, SPLNONE, DEFAULT);
	NLI("udb.inp_lock", inp_udphead_li, SPLNONE, DEFAULT);
	NLI("inifaddr_lock", inifaddr_li, SPLNONE, DEFAULT);
	NLI("igmp_lock", igmp_li, SPLNONE, DEFAULT);
	NLI("mrt_lock", mrt_li, SPLNONE, DEFAULT);
	NLI("route_lock", route_li, SPLNONE, DEFAULT);
	NLI("ip_frag_lock", ip_frag_li, SPLNONE, DEFAULT);
#if DLI > 0 || DLB > 0
	NLI("dli_dlif_lock", dli_dlif_li, SPLNONE, DEFAULT);
#endif
#if NTRA > 0 || TRSRCF > 0
	/* srt_thread_lock needs to be after dli_dlif_lock */
	NLI("srt_thread_lock", srtab_li, SPLNONE, DEFAULT);
#endif

#if NFAA > 0
	LI("fxx_softc.lk_thread_level",
		faa_thread_level_lockinfo, SPLNONE, DEFAULT);
#endif
#if DEC_AUDIT > 0
	ULI("audinit.lock", audinit_lockinfo, SPLNONE, DEFAULT);
#endif

	ULI("heap_lock", heap_lockinfo, SPLNONE, DEFAULT);
	LI("zone.z.complex_lock", zone_complex_lockinfo, SPLNONE, DEFAULT);
	LI("vm_kmap.vm_lock", k_map_lockinfo, SPLNONE, ORDERED);

#if NRPTY > 0
	LI("pty_allocation_lock", pty_opcl_lockinfo, SPLNONE, DEFAULT);
#endif
#if NSCSI > 0
	ULI("cam_pd_unit",cam_pd_unit_li,   SPLBIO, DEFAULT);
#if NSIOP > 0
	/* CAM siop */
	ULI("cam_siop_jalloc",  cam_siop_jalloc_li,     SPLBIO, DEFAULT);
	ULI("cam_siop_balloc",  cam_siop_balloc_li,     SPLBIO, DEFAULT);
#endif
#endif
	/*******************************************/
	/* end of lockinfo specs for complex locks */
	/*******************************************/
	/******************************************/
        /* start of lockinfo specs for NOCHECK    */
	/* complex locks.			  */
	/******************************************/
	ULI_NOCHECK("bp.b_lock", ubc_bp_lockinfo);
#if	(CDFS > 0)
	ULI_NOCHECK("cdnode.cd_io_lock", cdfs_inio_lockinfo);
#endif	/* CDFS > 0 */
	ULI_NOCHECK("bdevlock.dsw_lock", bdev_lockinfo);
	ULI_NOCHECK("cdevlock.dsw_lock", cdev_lockinfo);
	ULI_NOCHECK("handler_id_lock", handler_id_lockinfo);
	ULI_NOCHECK("handler_key.lock", handler_key_lockinfo);
	ULI_NOCHECK("ldbl_module_list_lock", ldbl_module_list_lockinfo);
	ULI_NOCHECK("topology_tree_lock", topology_tree_lockinfo);
#if	(NVBA > 0) || (CVBA > 0)
	ULI_NOCHECK("vhp_bus_adapt.shoebox_lk", vhp_shobox_lockinfo);
	ULI_NOCHECK("vhp_bus_adapt.adapt_c_lk", vhp_adapt_c_lockinfo);
#endif	/* (NVBA > 0) || (CVBA > 0) */
#if	(NLV > 0)
	ULI_NOCHECK("volgrp.vg_lock", vg_lockinfo);
	ULI_NOCHECK("lvol.lv_lock", lvtmp_lockinfo);
#endif	/* NLV > 0 */
#if	(NLSM_TED > 0)
	ULI_NOCHECK("ted_cmd_slepp_lock", ted_cmd_slepp_lockinfo);
#endif	/* (NLSM_TED > 0) */
	ULI_NOCHECK("ufsmount.um_lock", umpq_lockinfo);
	ULI_NOCHECK("dqhead.dq_lock", dq_lockinfo);
	ULI_NOCHECK("fifonode.fn_rdio_lock", fifo_rdio_lockinfo);
	ULI_NOCHECK("fifonode.fn_wrio_lock", fifo_wrio_lockinfo);
	ULI_NOCHECK("bp.b_lock", buf_lockinfo);
	ULI_NOCHECK("sleeplck.fichain_lock", sleeplcks_fichain_lockinfo);
	ULI_NOCHECK("grantlcks.fichain_lock", grantlcks_fichain_lockinfo);
	ULI_NOCHECK("klmlcks.fichain_lock", klmlcks_fichain_lock);
	ULI_NOCHECK("mount.m_unmount_lock", unmount_lockinfo);
	ULI_NOCHECK("vfssw_lock", vfssw_lockinfo);

#if	(NLSM > 0)
	ULI_NOCHECK("volcpybuf_sleep.sl_sleep", vol_alloc_sleep_lockinfo);
#endif	/* NLSM	> 0) */

	LI("lock.l_lock", interlock_lockinfo, SPLNONE, DEFAULT);
	LI_DISABLE_HIERARCHY_CHECKING(interlock_lockinfo);
	SLI("zone.z.lock", zone_simple_lockinfo, SPLNONE, DEFAULT);
	SLI_DISABLE_HIERARCHY_CHECKING(zone_simple_lockinfo);
	SLI("zone_page_table_lock", zone_page_table_lockinfo, SPLNONE, DEFAULT);
	SLI_DISABLE_HIERARCHY_CHECKING(zone_page_table_lockinfo);

	/********************************************/
	/* start of lockinfo specs for simple locks */
	/********************************************/

	ULI("pr_s5_dirlock", pr_s5_dir_lockinfo, SPLNONE, DEFAULT);
	ULI("pr_g_dirlock", pr_g_dir_lockinfo, SPLNONE, DEFAULT);

	SLI("hostname_lock", hostname_lockinfo, SPLNONE, DEFAULT);
	SLI("arptab_lock", arptab_li, SPLNONE, DEFAULT);
	NLI("inifaddr_spin_lock", inifaddr_spin_li, SPLNONE, DEFAULT);
	NLI("ifqueue.ifq_slock_splnone", ifqueue_splnone_li, SPLNONE, DEFAULT);
	NLI("domain.dom_rc_lock", dom_rc_li, SPLNONE, DEFAULT);
	NLI("global_unpconn_lock", global_unpconn_li, SPLNONE, DEFAULT);
#if	(DLB > 0)
	NLI("dlb_slock", dlb_sli, SPLNONE, DEFAULT);
	NLI("dlbq_lock", dlbq_sli, SPLNONE, DEFAULT);
#endif

	ULI("rpc_cl_xid_lock", rpc_cl_xid_lock_info, SPLNONE, DEFAULT);
	ULI("lk_rpcrqcred", lk_rpcrqcred_info, SPLNONE, DEFAULT);
	ULI("lk_rpccallout", lk_rpccallout_info, SPLNONE, DEFAULT);
	ULI("lk_rpcdupreq", lk_rpcdupreq_info, SPLNONE, DEFAULT);

	ULI("nfs_activewrites_lock",nfs_activewrites_lockinfo,SPLNONE,DEFAULT);
	ULI("nfs3_activewrites_lock",nfs3_activewrites_lockinfo,SPLNONE,DEFAULT);
	ULI("nfs_writefree_lock", nfs_writefree_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs3_writefree_lock", nfs3_writefree_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs_xprtfree_lock", nfs_xprtfree_lockinfo, SPLNONE, DEFAULT);
	ULI("nfs_sys_lock", nfs_sys_lockinfo, SPLNONE, DEFAULT);
	ULI("asyncdaemon_lock", asyncdaemon_lockinfo, SPLNONE, DEFAULT);
	ULI("presto_lock", presto_lockinfo, SPLNONE, DEFAULT);

	SLI("cur_msghdrs", cur_msghdrs_lockinfo, SPLNONE, DEFAULT);
	SLI("shm_stats_lock", shm_stats_lockinfo, SPLNONE, DEFAULT);
	SLI("all_zones_lock", all_zones_lockinfo, SPLNONE, DEFAULT);
	SLI("stack_queue_lock", stack_queue_lockinfo, SPLNONE, DEFAULT);
	SLI("all_psets_lock", all_psets_lockinfo, SPLNONE, DEFAULT);

	ULI("pgrp.pg_sig_lock", pgrp_sig_lockinfo, SPLNONE, DEFAULT);
	ULI("accounting_lock", accounting_lockinfo, SPLNONE, DEFAULT);
	ULI("aio_lock", aio_lockinfo, SPLNONE, DEFAULT);
	ULI("aioinit_lock", aioinit_lockinfo, SPLNONE, DEFAULT);
	ULI("gidhash_lock", gidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("uidhash_lock", uidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("ruidhash_lock", ruidhash_lockinfo, SPLNONE, DEFAULT);
	ULI("pid_lock", pid_lockinfo, SPLNONE, DEFAULT);
	ULI("proc.p_lock", proc_lockinfo, SPLNONE, DEFAULT);
	ULI("proc.p_sig_lock", proc_sig_lockinfo, SPLNONE, DEFAULT);
	ULI("session.s_lock", sess_lockinfo, SPLNONE, DEFAULT);

	SLI("task_swap_queues_lock",task_swap_queues_lockinfo,SPLNONE,DEFAULT);
	SLI("task.lock", task_lockinfo, SPLNONE, ORDERED);
	SLI("processor_set.lock", pset_lockinfo, SPLNONE, ORDERED);
	SLI("task.ipc_translation_lock", ipc_task_lockinfo, SPLNONE, DEFAULT);
	SLI("thread.ipc_state_lock", ipc_thread_lockinfo, SPLNONE, DEFAULT);
	SLI("kern_port.port_data_lock", kern_port_lockinfo, SPLNONE, DEFAULT);
	SLI("kern_set.set_data_lock", kern_set_lockinfo, SPLNONE, ORDERED);
	SLI("msg_queue.lock", msg_queue_lockinfo, SPLNONE, ORDERED);
	SLI("port_hash_bucket.lock", port_hash_lockinfo, SPLNONE, DEFAULT);
	SLI("reply_port_cache_lock", reply_pcache_lockinfo, SPLNONE, DEFAULT);
	SLI("task_ref_count_lock", task_ref_count_lockinfo, SPLNONE, DEFAULT);

	ULI("ufile_state.uf_ofile_lock", ufd_lockinfo, SPLNONE, DEFAULT);
	ULI("utask_nd.utnd_lock", utnd_lockinfo, SPLNONE, DEFAULT);
	ULI("ihash_lock", ihash_lockinfo, SPLNONE, DEFAULT);
	ULI("inode.i_incore_lock", i_incore_lockinfo, SPLNONE, ORDERED);
	ULI("utask.uu_handy_lock", u_handy_lockinfo, SPLNONE, DEFAULT);
	ULI("real_ucred.lock", ucred_lockinfo, SPLNONE, DEFAULT);
	ULI("file.f_incore_lock", file_lockinfo, SPLNONE, DEFAULT);
	ULI("vnode.v_lock", vnode_lockinfo, SPLNONE, DEFAULT);
	ULI("free_file_lock", free_file_lockinfo, SPLNONE, DEFAULT);
	ULI("namei.nch_lockinfo", nch_lockinfo, SPLNONE, DEFAULT);
	ULI("vn_free_lock", vn_free_lockinfo, SPLNONE, DEFAULT);
	ULI("nc_freelist_lockinfo", nc_freelist_lockinfo, SPLNONE, DEFAULT);
	ULI("nc_purge_lockinfo", nc_purge_lockinfo, SPLNONE, DEFAULT);
	NLI("unp_misc_lock", unp_misc_li, SPLNONE, DEFAULT);
	NLI("tcb.inp_rc_lock", inp_tcp_rc_li, SPLNONE, DEFAULT);
	NLI("udb.inp_rc_lock", inp_udp_rc_li, SPLNONE, DEFAULT);
	NLI("mrtlencap_lock", mrtlencap_li, SPLNONE, DEFAULT);
	NLI("mrtvcache_lock", mrtvcache_li, SPLNONE, DEFAULT);
#if LOCK_NETSTATS
	NLI("rtstat.rts_lock", rtstat_li, SPLNONE, DEFAULT);
	NLI("icmpstat.icps_lock", icmpstat_li, SPLNONE, DEFAULT);
	NLI("igmpstat.igps_lock", igmpstat_li, SPLNONE, DEFAULT);
	NLI("mrtstat.mrts_lock", mrtstat_li, SPLNONE, DEFAULT);
	NLI("ipstat.ips_lock", ipstat_li, SPLNONE, DEFAULT);
#endif
	NLI("misc_tcp_lock", misc_tcp_li, SPLNONE, DEFAULT);
#if LOCK_NETSTATS
	NLI("tcpstat.tcps_lock", tcpstat_li, SPLNONE, DEFAULT);
	NLI("udpstat.udps_lock", udpstat_li, SPLNONE, DEFAULT);
#endif
#if NPTY > 0
	ULI("pty_lock", pty_lockinfo, SPLNONE, DEFAULT);
#endif
#if NRPTY > 0
	SLI("pty_lock", pty_lockinfo, SPLNONE, DEFAULT);
#endif
#if DEC_AUDIT > 0
	ULI("audlog.lock", audlog_lockinfo, SPLNONE, DEFAULT);
	ULI("audselq.lock", audselq_lockinfo, SPLNONE, DEFAULT);
	ULI("audbuf.lock", audbuf_lockinfo, SPLNONE, DEFAULT);
	ULI("audmask.lock", audmask_lockinfo, SPLNONE, DEFAULT);
#endif

	SLI("consvp_lock", consvp_lockinfo, SPLNONE, DEFAULT);
	SLI("zget_space_lock", zget_space_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_map.ref_lock", vm_map_ref_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_map.hint_lock", vm_map_hint_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_map_entry.vme_faultlock", vme_fault_lockinfo, SPLNONE, DEFAULT);
	SLI("u_map_private.um_lock", ump_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_vp_object.vo_seglock", vop_seglockinfo, SPLNONE, DEFAULT);
	SLI("vm_object.ob_lock", vm_object_lockinfo, SPLNONE, ORDERED);
	SLI("ubc_dead_object_lock", ubc_dead_object_lockinfo, SPLNONE, DEFAULT);
	SLI("a_lock", a_lock_lockinfo, SPLNONE, DEFAULT);
	SLI("a_swaphash_lock", a_swaphash_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_circular_lock", vm_swap_circular_lockinfo,SPLNONE,DEFAULT);
	SLI("vm_swap.vs_lock", vm_swap_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_io_lock", vm_swap_io_lockinfo, SPLNONE, DEFAULT);
	SLI("h_kmem_wanted_lock", h_kmem_wanted_lockinfo, SPLNONE, DEFAULT);
	SLI("heap_list_lock", heap_list_lockinfo, SPLNONE, DEFAULT);
	SLI("heap_slock", heap_slockinfo, SPLNONE, DEFAULT);
	SLI("vm_swap_space_lock", vm_swap_space_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_queue_lock", vm_page_queue_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_lock_data", vm_page_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_wire_lock", vm_page_wire_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_lru_lock", ubc_lru_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_pages_lock", ubc_pages_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_pfree_lock", ubc_pfree_lockinfo, SPLNONE, DEFAULT);
	SLI("ubc_dirty_lock", ubc_dirty_lockinfo, SPLNONE, DEFAULT);
	SLI("u_seg_cache_lock", u_seg_cache_lockinfo, SPLNONE, DEFAULT);
	SLI("ptepages_lock", ptepages_lockinfo, SPLNONE, DEFAULT);
	SLI("anon_klock.akl_slock", akl_slock_lockinfo, SPLNONE, DEFAULT);
	SLI("vm_page_free_lock", vm_page_free_lockinfo, SPLNONE, DEFAULT);
	SLI("pmap_memdsc_lock", pmap_memdsc_lockinfo, SPLNONE, DEFAULT);

#ifdef STREAMS
	SLI("modsw_lock", modsw_lockinfo, SPLNONE, DEFAULT);
	SLI("tmtab_tb_lock", tmtab_tb_lockinfo, SPLNONE, DEFAULT);
	SLI("sth_pipestat.lock", stb_lockinfo, SPLNONE, DEFAULT);
	SLI("sad_lock", sad_lockinfo, SPLNONE, DEFAULT);
	SLI("muxid_lock", streams_muxid_lockinfo, SPLNONE, DEFAULT);
	SLI("iocblk_id_lock", iocblk_id_lockinfo, SPLNONE, DEFAULT);
	SLI("weldq_lock", weldq_lockinfo, SPLNONE, DEFAULT);
	SLI("bufc_lock",streams_bufc_lockinfo, SPLNONE, DEFAULT);
	SLI("active_qlock_lock", active_qlock_lockinfo, SPLNONE, DEFAULT);
	SLI("streams_open_lock", streams_open_lockinfo, SPLNONE, DEFAULT);
	SLI("sth_s.sth_ext_flags_lock", sth_ext_flags_lockinfo, SPLNONE, DEFAULT);
#endif	/* STREAMS */

#if XTISO
	SLI("xtiso_lock", xtiso_lockinfo, SPLNONE, DEFAULT);
	SLI("xtisoc_lock", xtisoc_lockinfo, SPLNONE, DEFAULT);
#endif	/* XTISO */

#if LDTTY
	SLI("ldtty.t_intimeout_lock", tp_intimeout_lockinfo, SPLNONE, DEFAULT);
#endif

#if RT_SEM
	SLI("real_time.p4bsem_lock", p4bsem_lockinfo, SPLNONE, DEFAULT);
#endif
#if RT_CSEM || RT_IPC
	SLI("real_time.p4csem_lock", p4csem_lockinfo, SPLNONE, DEFAULT);
#endif

	ULI("error_event_lock", event_cntrl_li, SPLSOFTC, DEFAULT);
#if DLPI
	NLI("dltimer_lock", dltimer_lockinfo, SPLSOFTC, DEFAULT);
#endif

#ifdef	ASE
	ULI("amlog",            amlog_lock_li,          SPLSOFTC, DEFAULT);
	ULI("amcntrl",          am_cntrl_lock_li,       SPLSOFTC, DEFAULT);
	ULI("amcntrl.hflink",   am_host_lk_li,          SPLSOFTC, DEFAULT);
	ULI("amcntrl.dflink",   am_device_lk_li,        SPLSOFTC, DEFAULT);
	ULI("ase_table",        ase_table_lock_li,      SPLSOFTC, DEFAULT);
#endif


	SLI("log_lock", log_lockinfo, SPL1, DEFAULT);
	NLI("ifqueue.ifq_slock_splnet", ifqueue_splnet_li, SPLNET, DEFAULT);

#if NSCSI > 0
	ULI("cam_pd_unit_table",cam_pd_unit_table_li,   SPLBIO, DEFAULT);
	ULI("cam_pd_device",    cam_pd_device_li,       SPLBIO, DEFAULT);
	ULI("cam_ccmn_bp_pool", cam_ccmn_bp_pool_li,    SPLBIO, DEFAULT);
	ULI("cam_ccmn_data_pool",cam_ccmn_data_pool_li, SPLBIO, DEFAULT);
	ULI("cam_c_ctrl",       cam_c_ctrl_li,          SPLBIO, DEFAULT);
	ULI("cam_c_qhead",      cam_c_qhead_li,         SPLBIO, DEFAULT);
	ULI("cam_c_edt",        cam_c_edt_li,           SPLBIO, DEFAULT);
	ULI("cam_d3_ctrl",      cam_d3_ctrl_li,         SPLBIO, DEFAULT);
	ULI("cam_x_qhead",      cam_x_qhead_li,         SPLBIO, DEFAULT);
	ULI("cam_x_ctrl",       cam_x_ctrl_li,          SPLBIO, DEFAULT);
	ULI("cam_reg",          cam_reg_li,             SPLBIO, DEFAULT);
	ULI("cam_softc",        cam_softc_li,           SPLBIO, DEFAULT);
	ULI("cam_reg",          cam_reg_li,             SPLBIO, DEFAULT);
	ULI("cam_cplt",         cam_cplt_li,            SPLBIO, DEFAULT);
	ULI("cam_sm",           cam_sm_li,              SPLBIO, DEFAULT);
#endif
#if NXZA > 0
	ULI("cam_xza_target",   cam_xza_target_li,      SPLBIO, DEFAULT);
#endif
#if NSIOP > 0
	ULI("cam_siop_doneq",   cam_siop_doneq_li,      SPLBIO, DEFAULT);
	ULI("cam_free_job",     cam_free_job_li,        SPLBIO, DEFAULT);
	ULI("cam_free_buf",     cam_free_buf_li,        SPLBIO, DEFAULT);
	ULI("cam_sh",           cam_sh_li,              SPLBIO, DEFAULT);
	ULI("cam_siopq",        cam_siopq_li,           SPLBIO, DEFAULT);
#endif
#if NPSIOP > 0
        ULI("cam_psiop_doneq",   cam_psiop_doneq_li,    SPLBIO, DEFAULT);
        ULI("cam_pfree_job",     cam_pfree_job_li,      SPLBIO, DEFAULT);
        ULI("cam_psh",           cam_psh_li,            SPLBIO, DEFAULT);
        ULI("cam_psiopq",        cam_psiopq_li,         SPLBIO, DEFAULT);
#endif
#if NAHA > 0
	ULI("cam_aha_softc",	 cam_aha_softc_li,	SPLBIO, DEFAULT);
	ULI("cam_aha_freeq",	 cam_aha_freeq_li,	SPLBIO, DEFAULT);
	ULI("cam_aha_activeq",	 cam_aha_activeq_li,	SPLBIO, DEFAULT);
#endif
#if NTZA > 0 || NPZA > 0
	ULI("cam_spo_generic",	 cam_spo_genericlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_resource",	 cam_spo_resourcelk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_nexus",	 cam_spo_nexuslk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_iowaitq",	 cam_spo_iowaitq_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_dacq",	 cam_spo_dacqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_dafq",	 cam_spo_dafqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_adrq",	 cam_spo_adrqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_adfq",	 cam_spo_adfqlk_li,	SPLBIO, DEFAULT);
	ULI("cam_spo_debug",	 cam_spo_debugqlk_li,	SPLBIO, DEFAULT);
#endif /* NTZA > 0 || NPZA > 0 */
#if NISP > 0 
	ULI("cam_isp",		 cam_isp_li,		SPLBIO, DEFAULT);
#endif /* NISP > 0 */
#if NXCR > 0
	ULI("astro_re_table",	re_table_li,		SPLBIO, DEFAULT);
	ULI("astro_re_device",	re_device_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_softc",	xcr_softc_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_reg",	xcr_reg_li,		SPLBIO, DEFAULT);
	ULI("astro_xcr_cb",	xcr_cb_li,		SPLBIO, DEFAULT);
#endif
	SLI("ubc_iodone_lock", ubc_iodone_lockinfo, SPLBIO, DEFAULT);
#ifdef STREAMS
	SLI("osrq_lock", osrq_lockinfo, SPLDEVHIGH,DEFAULT);
	SLI("streams_log_lock", streamsm_log_lockinfo, SPLDEVHIGH,DEFAULT);
	SLI("mblk_lock",streams_mblk_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("queue(streams).q_lock", streams_q_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("sqh_s.sqh_lock", sqh_lockinfo, SPLDEVHIGH, DEFAULT);
	SLI("str_to_lock", str_to_lockinfo, SPLDEVHIGH, DEFAULT);
#endif
#if NGWSCREEN > 0
	NLI("gwscreen_lock", gwscreen_li, SPLIMP, DEFAULT);
#endif
#if NTE > 0	/* TEGEC */
	SLI("te_softc.lk_te_softc", te_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NTU > 0	/* TULIP */
	SLI("tu_softc.lk_tu_softc", tu_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NXNA > 0	/* DEMNA */
	SLI("xna_softc.lk_xna_softc", xna_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NMFA > 0	/* DEMFA */
	SLI("mfa_softc.lk_mfa_softc", mfa_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NFTA > 0	/* DEFTA, DEFEA, DEFPA */
	SLI("fta_softc.lk_fta_softc", fta_softc_lockinfo, SPLIMP, DEFAULT);
#endif
#if NFAA > 0	/* DEFAA */
	SLI("fxx_softc.lk_interrupt_level",
		faa_interrupt_level_lockinfo, SPLIMP, DEFAULT);
#endif
#if PACKETFILTER > 0
	NLI("pfilt_lock", pfilt_lockinfo, SPLIMP, DEFAULT);
#endif
#ifdef	DEC2100_A500
	ULI("T2_hae0_3_lock",        T2_hae0_3_li, SPLDEVHIGH, DEFAULT);
#endif	/* NPCI	*/
#ifdef	DEC21000
	ULI("pcia_ctl_lock",        pcia_ctl_li, SPLDEVHIGH, DEFAULT);
#endif	/* DEC21000	*/
#if NPCI > 0
	ULI("dma_hw_sg_rmap_lock", sg_rmap_lockinfo, SPLDEVHIGH, DEFAULT);
#endif
	NLI("ifqueue.ifq_slock", ifqueue_li, SPLIMP, DEFAULT);
	NLI("ifnet.if_slock", ifnet_li, SPLIMP, DEFAULT);
	NLI("mbuf_slock", mbuf_sli, SPLIMP, DEFAULT);
	ULI("cku_private.lk_outbuf", cku_lk_outbuf_info, SPLIMP, DEFAULT);
	ULI("udp_data.lk_udpdata", lk_udpdata_info, SPLIMP, DEFAULT);
	ULI("nfs_rrok_lock", nfs_rrok_lockinfo, SPLIMP, DEFAULT);
	SLI("vm_swdone_lock", vm_swdone_lockinfo, SPLIMP, DEFAULT);
	SLI("vm_page_bucket.bk_lock", vm_page_bucket_lockinfo, SPLIMP, DEFAULT);
	NLI("netisr_slock", netisr_li, SPLIMP, DEFAULT);

	SLI("pmap_seg.ps_seg_lock", pmap_seg_ps_seglockinfo,
							SPLIMP, DEFAULT);
	SLI("pmap.lock", user_pmap_lockinfo, SPLIMP, DEFAULT);
	SLI("pmap.other_lock", user_pmap_other_lockinfo, SPLIMP, DEFAULT);
	SLI("kernel_pmap.lock", kernel_pmap_lockinfo, SPLIMP, DEFAULT);
	SLI("kernel_pmap.other_lock", kernel_pmap_other_lockinfo,
							SPLIMP, DEFAULT);
	SLI("kmemdata.waitlock", kmemdata_waitlock_info, SPLDEVHIGH, DEFAULT);
	SLI("kmembuckets.kb_lock", kdp_kb_lockinfo, SPLDEVHIGH, DEFAULT);

	SLI("binlog_selq_lock", binlog_selq_lockinfo, SPLHIGH, DEFAULT);
	SLI("task.thread_list_lock",task_thread_list_lockinfo,SPLSCHED,DEFAULT);
	SLI("reaper_lock", reaper_lockinfo, SPLSCHED, DEFAULT);
	SLI("event.ev_slock", event_lockinfo, SPLSCHED, DEFAULT);
	SLI("thread_callq.tcq_lock", tcq_lockinfo, SPLHIGH, DEFAULT);
	ULI("proc.p_timer_lock", proc_timer_lockinfo, SPLHIGH, DEFAULT);
	SLI("mpqueue_head.lock", mpqueue_lockinfo, SPLHIGH, DEFAULT);

	/* lock classes below here cannot use assert_wait, wakeup, etc. */

	SLI("wait_queue.lock", wait_queue_lockinfo, SPLSCHED, DEFAULT);
	SLI("thread.lock", thread_lockinfo, SPLSCHED, ORDERED);
	SLI("swapper_lock_data", swapper_lockinfo, SPLSCHED, DEFAULT);
	SLI("callout_lock", callout_lockinfo, SPLHIGH, DEFAULT);
	SLI("processor_set.idle_lock", pset_idle_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor.lock", processor_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor.runq.lock", proc_runq_lockinfo, SPLSCHED, DEFAULT);
	SLI("processor_set.runq.lock", pset_runq_lockinfo, SPLSCHED, DEFAULT);
	SLI("action_lock", action_lockinfo, SPLSCHED, DEFAULT);

	ULI("session.s_fpgrp_lock", sess_fpgrp_lockinfo, SPLHIGH, DEFAULT);
	ULI("pgrp.pg_ref_lock", pgrp_refcnt_lockinfo, SPLHIGH, DEFAULT);
	SLI("xprlock", xprlockinfo, SPLHIGH, DEFAULT);
	SLI("processor.callout_lock", processor_callout_lockinfo, SPLDEVRT, ORDERED);
	SLI("callout_free_lock", callout_free_lockinfo, SPLDEVRT, DEFAULT);

	SLI("binlog_lock", binlog_lockinfo, SPLEXTREME, DEFAULT);
	ULI("lwc_lock", lwc_lockinfo, SPLEXTREME, DEFAULT);
	SLI("msgbuf_lock", msgbuf_lockinfo, SPLEXTREME, DEFAULT);

	/******************************************/
	/* end of lockinfo specs for simple locks */
	/******************************************/

	/******************************************/
        /* start of lockinfo specs for NOCHECK    */
	/* simple locks.			  */
	/******************************************/

#if	(DEC4000 == 1)
	ULI_NOCHECK("kn430_fprint_lock", kn430_fprint_lockinfo);
	ULI_NOCHECK("kn430_watch_lock", kn430_watch_lockinfo);
#endif	/* DEC4000 */
#if	(NTRA > 0)
	ULI_NOCHECK("tra_softc.scb_clear_lock",  tra_lockinfo);
#endif 	/* (NTRA > 0 */
#if	MSFS > 0
	ULI_NOCHECK("LockMgrMutex.mutex",  msfs_lockmgrmutex_lockinfo);
	ULI_NOCHECK("mutexT.mutex",  msfs_mp_mutex_lockinfo);
#endif	/* MSFS > 0 */
	ULI_NOCHECK("bp_map.map_lock", bp_map_lockinfo);
	ULI_NOCHECK("cluster_bl_lock", cluster_bl_lockinfo);
	ULI_NOCHECK("exec_switch_lock", exec_switch_lockinfo);
#if	(NMLOG > 0)
	ULI_NOCHECK("mem_lock", mem_lockinfo);
#endif	/* NMLOG > 0 */
#if	(CDFS > 0)
	ULI_NOCHECK("cdfs.lock", cdfs_lockinfo);
	ULI_NOCHECK("cdhead.hash_lock", cdfs_hash_lockinfo);
	ULI_NOCHECK("cdfs.mount_lock", cdfsmount_lockinfo);
	ULI_NOCHECK("cnode.cd_incore_lock", cdfs_incore_lockinfo);
#endif	/* CDFS > 0 */
#if	(FFM_FS > 0)
	ULI_NOCHECK("ffm_mounttab_lock", ffm_mounttab_lockinfo);
#endif	/* FFM_FS > 0 */
#if	(NTCDS > 0)
	ULI_NOCHECK("DMA_CTRL.d3_lk_ctrl", d3cntrl_lockinfo);
#endif	/* NTCDS > 0 */
#if	(NPSIOP > 0)
	ULI_NOCHECK("siop_lock", siop_lockinfo);
#endif	/* NPSIOP > 0 */
	ULI_NOCHECK("nfs_export_lock", nfs_export_lockinfo);
#if	(VAGUE_STATS == 0)
	ULI_NOCHECK("rpc_cl_stat_lock", rpc_cl_stat_lockinfo);
	ULI_NOCHECK("rpc_sv_stat_lock", rpc_sv_stat_lockinfo);
#endif
#if	(NVBA > 0) || (CVBA > 0)
	ULI_NOCHECK("vba_bus_adapt.adapt_s_lock", vhp_adapt_s_lockinfo);
#endif	/* NVBA > 0 || CVBA > 0 */
#if	(STRIFNET > 0)
	ULI_NOCHECK("contextPtr.context_lock", streamsm_context_lockinfo);
	ULI_NOCHECK("list(streams)_lock", streamsm_list_lockinfo);
#endif	/* STRIFNET > 0 */
#if	(NLSM_TED > 0)
	ULI_NOCHECK("ted_lockp", ted_lockp_lockinfo);
	ULI_NOCHECK("ted_event.e_condp", ted_act_lockinfo);
#endif	/* NLSM_TED > 0 */
	ULI_NOCHECK("mfs_minor_lock", mfs_minor_lockinfo);
	ULI_NOCHECK("gen_lock", gen_lockinfo);
	ULI_NOCHECK("dhp.dqh_lock", dqh_lockinfo);
	ULI_NOCHECK("dqfree_lock", dqfree_lockinfo);
	ULI_NOCHECK("ump_qsync_lock", ump_qsync_lockinfo);
        ULI_NOCHECK("mounttab_lock", mounttab_lock);
	ULI_NOCHECK("fs.fs_lock", fs_lockinfo);
	ULI_NOCHECK("fifo_fileid_lock", fifo_fileid_lockinfo);
	ULI_NOCHECK("fifonode.fn_fifo_lock", fifo_lockinfo);
	ULI_NOCHECK("speclisth.sh_lock", spechash_lockinfo);
	ULI_NOCHECK("devlist.lock",devlist_lockinfo);
	ULI_NOCHECK("bfreelist_lock", bfreelist_lockinfo);
	ULI_NOCHECK("bufhd.bhd_lock", bhash_lockinfo);
	ULI_NOCHECK("flckinfo.flckinfo_lock", flckinfo_lockinfo);
	ULI_NOCHECK("flino.flino_lock", flino_lockinfo);
	ULI_NOCHECK("fids_lock", fids_lockinfo);
	ULI_NOCHECK("vn_dealloc_cache_lockup_ref",
		    vn_dealloc_cache_lookup_ref_lockinfo);
	ULI_NOCHECK("mount.m_lock", mount_lockinfo);
	ULI_NOCHECK("mount.m_vlist_lock", mount_vlist_lockinfo);
	ULI_NOCHECK("vnode.v_output_lock", vnode_output_lockinfo);
	ULI_NOCHECK("vnode.v_aux_lock", vnode_aux_lockinfo);
	ULI_NOCHECK("vnode.v_buflists_lock", buflists_lockinfo);
	ULI_NOCHECK("mountlist_lock", mountlist_lockinfo);
#if	MACH_LDEBUG
	ULI_NOCHECK("vnode_trace_lock", vnode_trace_lockinfo);
#endif	/* MACH_LDEBUG */
#if	(HEAP_TRACE > 0)
	ULI_NOCHECK("heap_trace_lock", heap_trace_lockinfo);
#endif	/* HEAP_TRACE > 0 */
#if	(MACH_XP_FPD > 0)
	ULI_NOCHECK("memory_manager_default_lock", 
	  	    memory_manager_default_lockinfo);
#endif	/* (MACH_XP_FPD > 0) */
	ULI_NOCHECK("vm_perf_lock", vm_perf_lockinfo);
	ULI_NOCHECK("vm_swap_list.vsl_lock", vm_swap_list_vsl_lockinfo);
#if	(NLSM > 0)
	ULI_NOCHECK("volspin.vol_alloc_spin_lock", vol_alloc_spin_lockinfo);
	ULI_NOCHECK("volsync.sy_lock", vol_allocsync_lockinfo);
#endif	/* NLSM	> 0)

	/******************************************/
        /* end of lockinfo specs for NOCHECK      */
	/* simple locks.			  */
	/******************************************/

	/******************************************/
        /* end of lockinfo specs for NOCHECK      */
	/* complex locks.			  */
	/******************************************/

	i = (1 << LOCKSTATS_SHIFT) - 1;
	lsp = (struct lockstats *)(((unsigned long)lip + i) & ~i);
	lockinfo_count = lip - lip_start;
	lockinfo_table = lip_start;
	for (i = 0; i < ncpus; i++, lsp += lockinfo_count)
		lockstats_table[i] = lsp;
	bzero((char *)lip, (char *)lsp - (char *)lip);
	return((char *)lsp - (char *)lip_start);
}
